#!/usr/bin/perl

use strict;
use warnings;

use DBI;

my $sqlite = shift;

die 'what file?' unless ( -e $sqlite );

my $dbh = DBI->connect("DBI:SQLite:dbname=$sqlite",'','')
	|| die 'failed to open db: ' . $DBI::errstr;

# create things we need
$dbh->do(q{
	CREATE INDEX
		IF NOT EXISTS
		exit_FromID
		ON ExitTbl ( FromID )
});

# convert colors from integer form to xterm256-rgb
&fixcolors();
# fix NULL names
$dbh->do(q{
	UPDATE ObjectTbl
		SET Name = ObjID
		WHERE Name IS NULL
});

my $dirhash = $dbh->selectall_hashref(q{
	SELECT DirID, DirName
		FROM DirTbl
},
	'DirID'
);

my $sth_exits = $dbh->prepare(q{
	SELECT ToID, DirType
		FROM ExitTbl
		WHERE FromID = ?
});

my %dirs = (
	'north' => ['n',  (1 << 0)],
	'east'  => ['e',  (1 << 1)],
	'south' => ['s',  (1 << 2)],
	'west'  => ['w',  (1 << 3)],
	'up'    => ['u',  (1 << 4)],
	'down'  => ['d',  (1 << 5)],
	'ne'    => ['ne', (1 << 6)],
	'nw'    => ['nw', (1 << 7)],
	'se'    => ['se', (1 << 8)],
	'sw'    => ['sw', (1 << 9)],
);

my $rooms = $dbh->selectall_arrayref(q{
	SELECT ObjID, Name, Color
		FROM ObjectTbl
});

foreach(@$rooms) {
	next unless ( defined $_->[1] );

	printf("R {%5i} {0} {%s} {%s} { }\n",
		$_->[0],
		$_->[2] || '<088>',
		$_->[1],
	);

	$sth_exits->execute($_->[0]) || die 'failed to execute exit query';

	my $exit;

	while ( $exit = $sth_exits->fetchrow_arrayref() ) {
		my $zdirnum = $exit->[1] + 1;
		my $dirname = $dirhash->{$zdirnum}->{'DirName'};
		my $ttdir   = $dirs{$dirname};

		printf("E {%5i} {%s} {%s} {%i}\n", $exit->[0], $ttdir->[0], $ttdir->[0], $ttdir->[1]);
	}

	print "\n";
}

$sth_exits->finish();

#$dbh->disconnect();

exit 0;

sub fixcolors {
	my $colors = $dbh->selectall_arrayref(q{
		SELECT Color
			FROM ObjectTbl
			GROUP BY Color
	});

	my $sth_fixcolors = $dbh->prepare(q{
		UPDATE ObjectTbl
			SET Color = ?
			WHERE Color = ?
	});

	foreach(@$colors) {
		if ( $_->[0] =~ /^\d+$/ ) {
			$sth_fixcolors->execute(
				'<' . &abgr2xterm256($_->[0]) . '>',
				$_->[0]
			);
		}
	}
}

sub abgr2xterm256 {
	my $num = 0 + shift;

#	my $a = $num / (256 ** 3) % 256;
	my $b = $num / (256 ** 2) % 256;
	my $g = $num / (256 ** 1) % 256;
	my $r = $num / (256 ** 0) % 256;

	$r = chr(ord('a') + &scale($r));
	$g = chr(ord('a') + &scale($g));
	$b = chr(ord('a') + &scale($b));

	return "$r$g$b";
}

sub scale {
	my $rgb256 = shift;

	int(($rgb256 / 255) * 5 + .5);
}
